part of io.repro.flutter.test;

class NewsFeedsTestRunner extends TestRunner {

  @override
  Future<void> run() async {

    await test("When limit: 0 is specified, an exception will be raised.", () async {
      return await Repro.getNewsFeeds(limit: 0, campaignType: NewsFeedCampaignType.All);
    }, (res) => res.hasException());

    await test("When limit: 1 is specified, an exception will not be raised.", () async {
      return await Repro.getNewsFeeds(limit: 1, campaignType: NewsFeedCampaignType.All);
    }, (res) => !res.hasException());

    await test("When offset: 0 is specified, an exception will be raised.", () async {
      return await Repro.getNewsFeeds(limit: 1, offsetID: 0, campaignType: NewsFeedCampaignType.All);
    }, (res) => res.hasException());

    await test("When offset: 1 is specified, an exception will not be raised.", () async {
      return await Repro.getNewsFeeds(limit: 1, offsetID: 1, campaignType: NewsFeedCampaignType.All);
    }, (res) => !res.hasException());

    await test("When type is set to null, an exception will be raised.", () async {
      return await Repro.getNewsFeeds(limit: 1, campaignType: null);
    }, (res) => res.hasException());

    await test("When PushNotification is specified as the type, only the list of pushes will be retrieved.", () async {
      return await Repro.getNewsFeeds(limit: 10, campaignType: NewsFeedCampaignType.PushNotification);
    }, (res) {
      var list = res.result as List<NewsFeedEntry>;
      return list.length > 0 && list.every((feed) => feed.campaignType == NewsFeedCampaignType.PushNotification);
    });

    await test("When InAppMessage is specified as the type, only the list of InApps will be retrieved.", () async {
      return await Repro.getNewsFeeds(limit: 10, campaignType: NewsFeedCampaignType.InAppMessage);
    }, (res) {
      var list = res.result as List<NewsFeedEntry>;
      return list.length > 0 && list.every((feed) => feed.campaignType == NewsFeedCampaignType.InAppMessage);
    });

    await test("When WebMessage is specified as the type, only the list of WebMessages will be retrieved.", () async {
      return await Repro.getNewsFeeds(limit: 10, campaignType: NewsFeedCampaignType.WebMessage);
    }, (res) {
      var list = res.result as List<NewsFeedEntry>;
      return list.length > 0 && list.every((feed) => feed.campaignType == NewsFeedCampaignType.WebMessage);
    });

    await test("When Unknown is specified as the type, an exception will be raised.", () async {
      return await Repro.getNewsFeeds(limit: 10, campaignType: NewsFeedCampaignType.Unknown);
    }, (res) {
      return res.hasException();
    });


    await test("When campaignType: all is set, each type of NewsFeeds will be retrieved.", () async {
      return await Repro.getNewsFeeds(limit: 10, campaignType: NewsFeedCampaignType.All);
    }, (res) {
      var list = res.result as List<NewsFeedEntry>;
      return list.any((feed) => feed.campaignType == NewsFeedCampaignType.PushNotification)
        && list.any((feed) => feed.campaignType == NewsFeedCampaignType.InAppMessage)
        && list.any((feed) => feed.campaignType == NewsFeedCampaignType.WebMessage);
    });

    await test("All NewsFeedEntry.campaignType in the results have a valid type.", () async {
      return await Repro.getNewsFeeds(limit: 10, campaignType: NewsFeedCampaignType.All);
    }, (res) {
      var list = res.result as List<NewsFeedEntry>;
      return list.any((element) => element.campaignType == NewsFeedCampaignType.PushNotification)
        && list.any((element) => element.campaignType == NewsFeedCampaignType.InAppMessage)
        && list.any((element) => element.campaignType == NewsFeedCampaignType.WebMessage);
    });

  }
}

