#import "ReproFlutterPlugin.h"
#import "Repro/Repro.h"
#import "Repro/RPRNewsFeedEntry.h"

#include <dispatch/dispatch.h>

@interface Repro (NonPublicApi)
+ (void)_passRuntimeValues:(nonnull NSDictionary<NSString *, NSString *> *)values;
@end

@implementation ReproFlutterPlugin

static NSMutableDictionary* callbackById;
static FlutterMethodChannel* channel;


+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    channel = [FlutterMethodChannel
               methodChannelWithName:@"io.repro.flutter"
               binaryMessenger:[registrar messenger]];
    ReproFlutterPlugin* instance = [[ReproFlutterPlugin alloc] init];
    [registrar addMethodCallDelegate:instance channel:channel];
    
    // Wait for initialization of MethodChannel on the dart side.
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(1.0 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [channel invokeMethod:@"passRuntimeValues_dart" arguments: @{}];
    });
}

// core/other

+ (NSString *)getUserID {
    return [Repro getUserID];
}

+ (void)setUserID:(id)arguments {
    NSString *userID = (NSString *)arguments;
    [Repro setUserID:userID];
}

+ (NSString *)getDeviceID {
    return [Repro getDeviceID];
}

+ (void)setLogLevel:(id)arguments {
    NSString *logLevel = (NSString *)arguments;
    if ([@"Debug" isEqualToString:logLevel]) {
        [Repro setLogLevel:RPRLogLevelDebug];
    } else if ([@"Info" isEqualToString:logLevel]) {
        [Repro setLogLevel:RPRLogLevelInfo];
    } else if ([@"Warn" isEqualToString:logLevel]) {
        [Repro setLogLevel:RPRLogLevelWarn];
    } else if ([@"Error" isEqualToString:logLevel]) {
        [Repro setLogLevel:RPRLogLevelError];
    }
}

+ (void)disableInAppMessagesOnForegroundTransition {
    [Repro disableInAppMessagesOnForegroundTransition];
}

+ (void)enableInAppMessagesOnForegroundTransition {
    [Repro enableInAppMessagesOnForegroundTransition];
}

+ (void)optIn:(id)arguments {
    [Repro optIn:[((NSNumber *)arguments) boolValue]];
}

// user profile

+ (void)setStringUserProfile:(id)arguments {
    NSArray *array = (NSArray *)arguments;
    NSString *key = (NSString *)array[0];
    NSString *value = (NSString *)array[1];
    [Repro setStringUserProfile:value forKey:key];
}

+ (void)setIntUserProfile:(id)arguments {
    NSArray *array = (NSArray *)arguments;
    NSString *key = (NSString *)array[0];
    NSNumber *value = (NSNumber *)array[1];
    [Repro setIntUserProfile:value.integerValue forKey:key];
}

+ (void)setDoubleUserProfile:(id)arguments {
    NSArray *array = (NSArray *)arguments;
    NSString *key = (NSString *)array[0];
    NSNumber *value = (NSNumber *)array[1];
    [Repro setDoubleUserProfile:value.doubleValue forKey:key];
}

+ (void)setDateUserProfile:(id)arguments {
    NSArray *array = (NSArray *)arguments;
    NSString *key = (NSString *)array[0];
    NSNumber *value = (NSNumber *)array[1];
    NSDate* date = [NSDate dateWithTimeIntervalSince1970:(value.longValue) / 1000.0];
    [Repro setDateUserProfile:date forKey:key];
}

+ (void)setUserGender:(id)arguments {
    NSString *gender = (NSString *)arguments;
    if ([@"Male" isEqualToString:gender]) {
        [Repro setUserGender:RPRUserProfileGenderMale];
    } else if ([@"Female" isEqualToString:gender]) {
        [Repro setUserGender:RPRUserProfileGenderFemale];
    } else if ([@"Other" isEqualToString:gender]) {
        [Repro setUserGender:RPRUserProfileGenderOther];
    }
}

+ (void)setUserEmailAddress:(id)arguments {
    NSString *address = (NSString *)arguments;
    [Repro setUserEmailAddress:address];
}

// event tracking

+ (void)track:(id)arguments {
    NSArray *array = (NSArray *)arguments;
    NSString *name = (NSString *)array[0];
    NSDictionary *dict = (NSDictionary *)array[1];

    [Repro track:name properties:dict];
}

+ (void)trackViewContent:(id)arguments {
    NSArray *array = (NSArray *)arguments;
    NSString *contentId = (NSString *)array[0];
    NSDictionary *dict = (NSDictionary *)array[1];
    RPRViewContentProperties *props = [[RPRViewContentProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"value"];               if ([tmp isKindOfClass:NSNumber.class])     { [props setValue:((NSNumber *)tmp).doubleValue]; }
    tmp = dict[@"currency"];            if ([tmp isKindOfClass:NSString.class])     { [props setCurrency:(NSString *)tmp]; }
    tmp = dict[@"content_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setContentName:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackViewContent:contentId properties:props];
}

+ (void)trackSearch:(id)arguments {
    NSDictionary *dict = (NSDictionary *)arguments;
    RPRSearchProperties *props = [[RPRSearchProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"content_id"];          if ([tmp isKindOfClass:NSString.class])     { [props setContentID:(NSString *)tmp]; }
    tmp = dict[@"value"];               if ([tmp isKindOfClass:NSNumber.class])     { [props setValue:((NSNumber *)tmp).doubleValue]; }
    tmp = dict[@"currency"];            if ([tmp isKindOfClass:NSString.class])     { [props setCurrency:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"search_string"];       if ([tmp isKindOfClass:NSString.class])     { [props setSearchString:(NSString *)tmp]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackSearch:props];
}

+ (void)trackAddToWishlist:(id)arguments {
    NSDictionary *dict = (NSDictionary *)arguments;
    RPRAddToWishlistProperties *props = [[RPRAddToWishlistProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"content_id"];          if ([tmp isKindOfClass:NSString.class])     { [props setContentID:(NSString *)tmp]; }
    tmp = dict[@"value"];               if ([tmp isKindOfClass:NSNumber.class])     { [props setValue:((NSNumber *)tmp).doubleValue]; }
    tmp = dict[@"currency"];            if ([tmp isKindOfClass:NSString.class])     { [props setCurrency:(NSString *)tmp]; }
    tmp = dict[@"content_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setContentName:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackAddToWishlist:props];
}

+ (void)trackAddToCart:(id)arguments {
    NSArray *array = (NSArray *)arguments;
    NSString *contentId = (NSString *)array[0];
    NSDictionary *dict = (NSDictionary *)array[1];
    RPRAddToCartProperties *props = [[RPRAddToCartProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"value"];               if ([tmp isKindOfClass:NSNumber.class])     { [props setValue:((NSNumber *)tmp).doubleValue]; }
    tmp = dict[@"currency"];            if ([tmp isKindOfClass:NSString.class])     { [props setCurrency:(NSString *)tmp]; }
    tmp = dict[@"content_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setContentName:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackAddToCart:contentId properties:props];
}

+ (void)trackInitiateCheckout:(id)arguments {
    NSDictionary *dict = (NSDictionary *)arguments;
    RPRInitiateCheckoutProperties *props = [[RPRInitiateCheckoutProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"content_id"];          if ([tmp isKindOfClass:NSString.class])     { [props setContentID:(NSString *)tmp]; }
    tmp = dict[@"value"];               if ([tmp isKindOfClass:NSNumber.class])     { [props setValue:((NSNumber *)tmp).doubleValue]; }
    tmp = dict[@"currency"];            if ([tmp isKindOfClass:NSString.class])     { [props setCurrency:(NSString *)tmp]; }
    tmp = dict[@"content_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setContentName:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"num_items"];           if ([tmp isKindOfClass:NSNumber.class])     { [props setNumItems:((NSNumber *)tmp).integerValue]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackInitiateCheckout:props];
}

+ (void)trackAddPaymentInfo:(id)arguments {
    NSDictionary *dict = (NSDictionary *)arguments;
    RPRAddPaymentInfoProperties *props = [[RPRAddPaymentInfoProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"content_id"];          if ([tmp isKindOfClass:NSString.class])     { [props setContentID:(NSString *)tmp]; }
    tmp = dict[@"value"];               if ([tmp isKindOfClass:NSNumber.class])     { [props setValue:((NSNumber *)tmp).doubleValue]; }
    tmp = dict[@"currency"];            if ([tmp isKindOfClass:NSString.class])     { [props setCurrency:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackAddPaymentInfo:props];
}

+ (void)trackPurchase:(id)arguments {
    NSArray *array = (NSArray *)arguments;
    NSString *contentId = (NSString *)array[0];
    NSNumber *value = (NSNumber *)array[1];
    NSString *currency = (NSString *)array[2];
    NSDictionary *dict = (NSDictionary *)array[3];
    RPRPurchaseProperties *props = [[RPRPurchaseProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"content_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setContentName:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"num_items"];           if ([tmp isKindOfClass:NSNumber.class])     { [props setNumItems:((NSNumber *)tmp).integerValue]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackPurchase:contentId value:value.doubleValue currency:currency properties:props];
}

+ (void)trackShare:(id)arguments {
    NSDictionary *dict = (NSDictionary *)arguments;
    RPRShareProperties *props = [[RPRShareProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"content_id"];          if ([tmp isKindOfClass:NSString.class])     { [props setContentID:(NSString *)tmp]; }
    tmp = dict[@"service_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setServiceName:(NSString *)tmp]; }
    tmp = dict[@"content_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setContentName:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackShare:props];
}

+ (void)trackCompleteRegistration:(id)arguments {
    NSDictionary *dict = (NSDictionary *)arguments;
    RPRCompleteRegistrationProperties *props = [[RPRCompleteRegistrationProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"value"];               if ([tmp isKindOfClass:NSNumber.class])     { [props setValue:((NSNumber *)tmp).doubleValue]; }
    tmp = dict[@"currency"];            if ([tmp isKindOfClass:NSString.class])     { [props setCurrency:(NSString *)tmp]; }
    tmp = dict[@"content_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setContentName:(NSString *)tmp]; }
    tmp = dict[@"status"];              if ([tmp isKindOfClass:NSString.class])     { [props setStatus:(NSString *)tmp]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackCompleteRegistration:props];
}

+ (void)trackLead:(id)arguments {
    NSDictionary *dict = (NSDictionary *)arguments;
    RPRLeadProperties *props = [[RPRLeadProperties alloc] init];
    NSObject *tmp;

    tmp = dict[@"value"];               if ([tmp isKindOfClass:NSNumber.class])     { [props setValue:((NSNumber *)tmp).doubleValue]; }
    tmp = dict[@"currency"];            if ([tmp isKindOfClass:NSString.class])     { [props setCurrency:(NSString *)tmp]; }
    tmp = dict[@"content_name"];        if ([tmp isKindOfClass:NSString.class])     { [props setContentName:(NSString *)tmp]; }
    tmp = dict[@"content_category"];    if ([tmp isKindOfClass:NSString.class])     { [props setContentCategory:(NSString *)tmp]; }
    tmp = dict[@"extras"];              if ([tmp isKindOfClass:NSDictionary.class]) { [props setExtras:(NSDictionary *)tmp]; }

    [Repro trackLead:props];
}

+ (void)setSilverEggCookie:(id)arguments {
    NSString* cookie = (NSString*)arguments;
    [Repro setSilverEggCookie:cookie];
}

+ (void)setSilverEggProdKey:(id)arguments {
    NSString* prodKey = (NSString*)arguments;
    [Repro setSilverEggProdKey:prodKey];
}

// Normally, this returns NSArray<NSDictionary *> *
+ (id)getNewsFeeds:(id)arguments {

    NSArray* array = (NSArray *)arguments;
    NSNumber* numLimit = array[0];
    NSNumber* numOffset = array[1];
    NSString* strCampaignType = array[2];
    
    int64_t limit = [numLimit longLongValue];
    
    RPRCampaignType campaignType = [self getNewsFeedCampaignType:strCampaignType];

    NSArray<RPRNewsFeedEntry *> * newsFeeds = nil;
    NSError* error = nil;

    if (numOffset && ![numOffset isEqual:[NSNull null]]) {
        int64_t offset = [numOffset longLongValue];
        newsFeeds = [Repro getNewsFeeds:limit offsetID:offset campaignType:campaignType error:&error];
    }
    else {
        newsFeeds = [Repro getNewsFeeds:limit campaignType:campaignType error:&error];
    }

    if (error) {
        return [FlutterError errorWithCode:@"Native Error" message:@"An error occurred in 'getNewsFeeds'" details:error.localizedDescription];
    }

    NSMutableArray<NSDictionary *> * arrayDic = [[NSMutableArray alloc] init];
    for (RPRNewsFeedEntry* entry in newsFeeds) {
        NSDictionary* dic = [ReproFlutterPlugin convertToDictionary:entry];
        [arrayDic addObject: dic];
    }
    return arrayDic;
}

+ (RPRCampaignType)getNewsFeedCampaignType:(nullable NSString *)rawCampaignType
{
    if ([rawCampaignType isKindOfClass:NSString.class]) {
        if ([rawCampaignType isEqualToString:@"push_notification"]) {
            return RPRCampaignTypePushNotification;
        }
        if ([rawCampaignType isEqualToString:@"in_app_message"]) {
            return RPRCampaignTypeInAppMessage;
        }
        if ([rawCampaignType isEqualToString:@"web_message"]) {
            return RPRCampaignTypeWebMessage;
        }
        if ([rawCampaignType isEqualToString:@"all"]) {
            return RPRCampaignTypeAll;
        }
    }
    return RPRCampaignTypeUnknown;
}

+ (NSString *)convertCampaignTypeToString:(RPRCampaignType)campaignType
{
    if (campaignType == RPRCampaignTypePushNotification) {
        return @"push_notification";
    }
    else if (campaignType == RPRCampaignTypeInAppMessage) {
        return @"in_app_message";
    }
    else if (campaignType == RPRCampaignTypeWebMessage) {
        return @"web_message";
    }
    return @"unknown";
}

+ (id)updateNewsFeeds:(id)arguments {
    NSArray<NSDictionary*>* stringMapList = (NSArray<NSDictionary*>*)arguments;

    NSMutableArray<RPRNewsFeedEntry*>* updateTargets = [[NSMutableArray alloc] init];
    for (NSDictionary* dic in stringMapList) {
        RPRNewsFeedEntry* entry = [[RPRNewsFeedEntry alloc] initWithDictionary:dic];
        [updateTargets addObject:entry];
    }

    NSError* error = nil;
    [Repro updateNewsFeeds:updateTargets error:&error];
    if (error) {
        return [FlutterError errorWithCode:@"Native Error" message:@"An error occurred in 'updateNewsFeeds'" details:error.localizedDescription];
    } else {
        return nil;
    }
}

+ (NSDictionary*)remoteConfig_get:(id)arguments {
    NSString* key = (NSString*)arguments;
    RPRRemoteConfig* remoteConfig = [Repro remoteConfig];
    RPRRemoteConfigValue* remoteConfigValue = [remoteConfig valueForKey:key];
    return [self convertRemoteConfigValueToDictionary:remoteConfigValue];
}

+ (NSDictionary*)remoteConfig_getAllValues:(id)arguments {
    RPRRemoteConfig* remoteConfig = [Repro remoteConfig];
    NSDictionary<NSString*, RPRRemoteConfigValue*>* result = [remoteConfig allValues];

    NSDictionary* newDic = [NSMutableDictionary dictionary];
    for (NSString* key in [result keyEnumerator]) {
        NSDictionary* convertedValue = [self convertRemoteConfigValueToDictionary:[result valueForKey:key]];
        [newDic setValue:convertedValue forKey:key];
    }
    return newDic;
}

+ (NSDictionary*)remoteConfig_getAllValuesWithPrefix:(id)arguments {
    NSString* prefix = (NSString*)arguments;
    
    RPRRemoteConfig* remoteConfig = [Repro remoteConfig];
    NSDictionary<NSString*, RPRRemoteConfigValue*>* result = [remoteConfig allValuesWithPrefix:prefix];

    NSDictionary* newDic = [NSMutableDictionary dictionary];
    for (NSString* key in [result keyEnumerator]) {
        NSDictionary* convertedValue = [self convertRemoteConfigValueToDictionary:[result valueForKey:key]];
        [newDic setValue:convertedValue forKey:key];
    }
    return newDic;
}


+ (NSDictionary*)convertRemoteConfigValueToDictionary:(RPRRemoteConfigValue*) remoteConfigValue {
    NSString* str = [remoteConfigValue stringValue];
    NSDictionary* dic = @{
        @"String": str,
        @"toString": str == nil ? @"" : str
    };
    return dic;
}

+ (void)remoteConfig_setDefaultsFromMap:(id)arguments {
    NSDictionary* defaultsDictionary = (NSDictionary*)arguments;
    RPRRemoteConfig* remoteConfig = [Repro remoteConfig];
    [remoteConfig setDefaultsFromDictionary:defaultsDictionary];
}

+ (void)remoteConfig_setDefaultsFromJsonString:(id)arguments {
    NSString* jsonString = (NSString*)arguments;
    RPRRemoteConfig* remoteConfig = [Repro remoteConfig];
    [remoteConfig setDefaultsFromJsonString:jsonString];
}

+ (BOOL)remoteConfig_activateFetched:(id)arguments {
    RPRRemoteConfig* remoteConfig = [Repro remoteConfig];
    return [remoteConfig activateFetched];
}

+ (void)remoteConfig_forceReset:(id)arguments {
    RPRRemoteConfig* remoteConfig = [Repro remoteConfig];
    [remoteConfig forceReset];
}

+ (NSDictionary*)remoteConfig_getLocalDefaultValue:(id)arguments {
    NSString* key = (NSString*)arguments;
    RPRRemoteConfig* remoteConfig = [Repro remoteConfig];
    RPRRemoteConfigValue* value = [remoteConfig localDefaultValueForKey:key];
    return [self convertRemoteConfigValueToDictionary:value];
}

+ (void)startListening_fetch:(id)arguments {
    NSArray* array = (NSArray *)arguments;
    NSString* currentListenerId = [(NSNumber*)array[0] stringValue];
    NSNumber* timeoutMillisecs = (NSNumber*)array[1];
    double timeoutSecs = [timeoutMillisecs longLongValue] * 0.001;
    
    if (!callbackById) {
        callbackById = [NSMutableDictionary new];
    }
    
    RPRRemoteConfig* remoteConfig = [Repro remoteConfig];
    
    [remoteConfig fetchWithTimeout:timeoutSecs completionHandler:^(RPRRemoteConfigFetchStatus status) {
        NSNumber* stat = [NSNumber numberWithInteger:status];
        
        [callbackById setObject:^() {
            if ([callbackById valueForKey:currentListenerId] != nil) {
                [channel invokeMethod:@"callListener"
                            arguments:@{
                                @"id" : [NSNumber numberWithInt:[currentListenerId intValue]],
                                @"args" : stat
                            }
                ];
            }
        } forKey:currentListenerId];
        void (^cb)(void) = [callbackById valueForKey:currentListenerId];
        cb();
    }];
}

+ (void)cancelListening:(id)arguments {
    NSString* listenerId = [(NSNumber*)arguments stringValue];
    [callbackById removeObjectForKey:listenerId];
}

+ (NSDictionary*)convertToDictionary:(RPRNewsFeedEntry*)newsFeedEntry {

    NSString* strId = @(newsFeedEntry.ID).stringValue;
    
    NSURL* linkUrl = newsFeedEntry.linkUrl;
    NSString* linkUrlString = @"";
    if (linkUrl != nil && [linkUrl absoluteString] != nil) {
        linkUrlString = [linkUrl absoluteString];
    }

    NSURL* imageUrl = newsFeedEntry.imageUrl;
    NSString* imageUrlString = @"";
    if (imageUrl != nil && [imageUrl absoluteString] != nil) {
        imageUrlString = [imageUrl absoluteString];
    }

    NSDateFormatter* formatter = NSDateFormatter.alloc.init;
    formatter.locale           = [NSLocale.alloc initWithLocaleIdentifier:@"en_US_POSIX"];
    formatter.calendar         = [NSCalendar.alloc initWithCalendarIdentifier:NSCalendarIdentifierGregorian];
    formatter.dateFormat       = @"yyyy-MM-dd'T'HH:mm:ssZZZ";
    NSString* timeStamp        = [formatter stringFromDate:newsFeedEntry.deliveredAt];

    NSString* shownString = newsFeedEntry.shown ? @"true" : @"false";
    NSString* readString = newsFeedEntry.read ? @"true" : @"false";

    NSDictionary* dic = @{
        @"id": strId,
        @"campaignType": [self convertCampaignTypeToString:newsFeedEntry.campaignType],
        @"deviceID": newsFeedEntry.deviceID,
        @"title": newsFeedEntry.title,
        @"summary": newsFeedEntry.summary,
        @"body": newsFeedEntry.body,
        @"linkUrl": linkUrlString,
        @"imageUrl": imageUrlString,
        @"deliveredAt": timeStamp,
        @"shown": shownString,
        @"read": readString
    };
    return dic;
}

+ (void)passRuntimeValues:(id)arguments {
    NSDictionary<NSString *, NSString *> *dic = (NSDictionary<NSString *, NSString *> *)arguments;
    [Repro _passRuntimeValues:dic];
}

- (void)handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    @try {
        // core/other
        if ([@"getUserID" isEqualToString:call.method]) {
            NSString *userID = [ReproFlutterPlugin getUserID];
            result(userID);
        } else if ([@"setUserID" isEqualToString:call.method]) {
            [ReproFlutterPlugin setUserID:call.arguments];
            result(nil);
        } else if ([@"getDeviceID" isEqualToString:call.method]) {
            NSString *deviceID = [ReproFlutterPlugin getDeviceID];
            result(deviceID);
        } else if ([@"setLogLevel" isEqualToString:call.method]) {
            [ReproFlutterPlugin setLogLevel:call.arguments];
            result(nil);
        } else if ([@"disableInAppMessagesOnForegroundTransition" isEqualToString:call.method]) {
            [ReproFlutterPlugin disableInAppMessagesOnForegroundTransition];
            result(nil);
        } else if ([@"enableInAppMessagesOnForegroundTransition" isEqualToString:call.method]) {
            [ReproFlutterPlugin enableInAppMessagesOnForegroundTransition];
            result(nil);
        } else if ([@"optIn" isEqualToString:call.method]) {
            [ReproFlutterPlugin optIn:call.arguments];
            result(nil);
        }

        // user profile

        else if ([@"setStringUserProfile" isEqualToString:call.method]) {
            [ReproFlutterPlugin setStringUserProfile:call.arguments];
            result(nil);
        } else if ([@"setIntUserProfile" isEqualToString:call.method]) {
            [ReproFlutterPlugin setIntUserProfile:call.arguments];
            result(nil);
        } else if ([@"setDoubleUserProfile" isEqualToString:call.method]) {
            [ReproFlutterPlugin setDoubleUserProfile:call.arguments];
            result(nil);
        } else if ([@"setDateUserProfile" isEqualToString:call.method]) {
            [ReproFlutterPlugin setDateUserProfile:call.arguments];
            result(nil);
        } else if ([@"setUserGender" isEqualToString:call.method]) {
            [ReproFlutterPlugin setUserGender:call.arguments];
            result(nil);
        } else if ([@"setUserEmailAddress" isEqualToString:call.method]) {
            [ReproFlutterPlugin setUserEmailAddress:call.arguments];
            result(nil);
        }

        // event tracking

        else if ([@"track" isEqualToString:call.method]) {
            [ReproFlutterPlugin track:call.arguments];
            result(nil);
        } else if ([@"trackViewContent" isEqualToString:call.method]) {
            [ReproFlutterPlugin trackViewContent:call.arguments];
            result(nil);
        } else if ([@"trackSearch" isEqualToString:call.method]) {
            [ReproFlutterPlugin trackSearch:call.arguments];
            result(nil);
        } else if ([@"trackAddToWishlist" isEqualToString:call.method]) {
            [ReproFlutterPlugin trackAddToWishlist:call.arguments];
            result(nil);
        } else if ([@"trackAddToCart" isEqualToString:call.method]) {
            [ReproFlutterPlugin trackAddToCart:call.arguments];
            result(nil);
        } else if ([@"trackInitiateCheckout" isEqualToString:call.method]) {
            [ReproFlutterPlugin trackInitiateCheckout:call.arguments];
            result(nil);
        } else if ([@"trackAddPaymentInfo" isEqualToString:call.method]) {
            [ReproFlutterPlugin trackAddPaymentInfo:call.arguments];
            result(nil);
        } else if ([@"trackPurchase" isEqualToString:call.method]) {
            [ReproFlutterPlugin trackPurchase:call.arguments];
            result(nil);
        } else if ([@"trackShare" isEqualToString:call.method]) {
            [ReproFlutterPlugin trackShare:call.arguments];
            result(nil);
        } else if ([@"trackCompleteRegistration" isEqualToString:call.method]) {
            [ReproFlutterPlugin trackCompleteRegistration:call.arguments];
            result(nil);
        } else if ([@"trackLead" isEqualToString:call.method]) {
            [ReproFlutterPlugin trackLead:call.arguments];
            result(nil);
        } else if ([@"getNewsFeeds" isEqualToString:call.method]) {
            // NSArray<NSDictionary*>* or FlutterError
            id newsFeeds = [ReproFlutterPlugin getNewsFeeds:call.arguments];
            result(newsFeeds); // When newsFeeds is FlutterError, an exception will be raised in MethodChannel#invoke.
        } else if ([@"updateNewsFeeds" isEqualToString:call.method]) {
            id error = [ReproFlutterPlugin updateNewsFeeds:call.arguments];
            result(error);
        } else if ([@"setSilverEggCookie" isEqualToString:call.method]) {
            [ReproFlutterPlugin setSilverEggCookie:call.arguments];
            result(nil);
        } else if ([@"setSilverEggProdKey" isEqualToString:call.method]) {
            [ReproFlutterPlugin setSilverEggProdKey:call.arguments];
            result(nil);
        } else if ([@"remoteConfig_get" isEqualToString:call.method]) {
            NSDictionary* dic = [ReproFlutterPlugin remoteConfig_get:call.arguments];
            result(dic);
        } else if ([@"remoteConfig_getAllValues" isEqualToString:call.method]) {
            NSDictionary* dic = [ReproFlutterPlugin remoteConfig_getAllValues:call.arguments];
            result(dic);
        } else if ([@"remoteConfig_getAllValuesWithPrefix" isEqualToString:call.method]) {
            NSDictionary* dic = [ReproFlutterPlugin remoteConfig_getAllValuesWithPrefix:call.arguments];
            result(dic);
        } else if ([@"remoteConfig_setDefaultsFromMap" isEqualToString:call.method]) {
            [ReproFlutterPlugin remoteConfig_setDefaultsFromMap:call.arguments];
            result(nil);
        } else if ([@"remoteConfig_setDefaultsFromJsonString" isEqualToString:call.method]) {
            [ReproFlutterPlugin remoteConfig_setDefaultsFromJsonString:call.arguments];
            result(nil);
        } else if ([@"remoteConfig_activateFetched" isEqualToString:call.method]) {
            BOOL resultValue = [ReproFlutterPlugin remoteConfig_activateFetched:call.arguments];
            result([NSNumber numberWithBool:resultValue]);
        } else if ([@"remoteConfig_forceReset" isEqualToString:call.method]) {
            [ReproFlutterPlugin remoteConfig_forceReset:call.arguments];
            result(nil);
        } else if ([@"remoteConfig_getLocalDefaultValue" isEqualToString:call.method]) {
            NSDictionary* dic = [ReproFlutterPlugin remoteConfig_getLocalDefaultValue:call.arguments];
            result(dic);
        } else if ([@"startListening_remoteConfig_fetch" isEqualToString:call.method]) {
            [ReproFlutterPlugin startListening_fetch:call.arguments];
            result(nil);
        } else if ([@"cancelListening" isEqualToString:call.method]) {
            [ReproFlutterPlugin cancelListening:call.arguments];
            result(nil);
        } else if ([@"passRuntimeValues_native" isEqualToString:call.method]) {
            [ReproFlutterPlugin passRuntimeValues:call.arguments];
            result(nil);
        }
        else {
            result(FlutterMethodNotImplemented);
        }
    }
    @catch (NSException* exception) {
        result([FlutterError errorWithCode:@"Native Error" message:exception.name details:exception.reason]);
    }
}

@end
